 /* -*- Mode: c -*-

     Lexer for the minimal programming language nanoLang. This version
     uses a yylex() function that returns after each token and must be
     called until End Of File (EOF) is encountered.

     Copyright 2017 by Stephan Schulz, schulz@eprover.org.

     This code is released under the GNU General Public Licence
     Version 2.
 */

   /* We don't provide a handler for EOF - we only compile one file. */

%option noyywrap

   /* We want to keep track of linen numbers automatically. */

%option yylineno

%{
#include <string.h>

   /* List of possible tokens. If used with Bison, this list comes
    * from the Bison-generated something.tab.h. By default, yylex()
    * will return 0 at the end of the file, so we give that a name as
    * well.  */

enum yytokentype
{
   EOFTok,
   OPENPAR,
   CLOSEPAR,
   MULT,
   DIV,
   PLUS,
   MINUS,
   EQ,
   NEQ,
   LT,
   GT,
   LEQ,
   GEQ,
   OPENCURLY,
   CLOSECURLY,
   SEMICOLON,
   COMMA,
   IDENT,
   STRINGLIT,
   INTLIT,
   INTEGER,
   STRING,
   IF,
   WHILE,
   RETURN,
   PRINT,
   ELSE,
   ERROR
};

   /* Printable names of the tokens. This is just a convenient hack to
    * be able to print the names without much work. */

char* tokname[] =
{
   "EOF",
   "OPENPAR",
   "CLOSEPAR",
   "MULT",
   "DIV",
   "PLUS",
   "MINUS",
   "EQ",
   "NEQ",
   "LT",
   "GT",
   "LEQ",
   "GEQ",
   "OPENCURLY",
   "CLOSECURLY",
   "SEMICOLON",
   "COMMA",
   "IDENT",
   "STRINGLIT",
   "INTLIT",
   "INTEGER",
   "STRING",
   "IF",
   "WHILE",
   "RETURN",
   "PRINT",
   "ELSE",
   "ERROR"
};

   /* We want to keep track of the columns, too */

int yycol = 0;

  /* Function to print a token - indeed, to print the current token,
   * since all the magic "yy" variables only apply to the current
   * token. */

void print_tok(int tok)
{
   printf("<%5d:%3d: %s[%d]=\"%s\">\n", yylineno, yycol-yyleng, tokname[tok], tok, yytext);
}

#define YY_USER_ACTION {yycol += yyleng;}


%}

DIGIT      [0-9]
ALPHA      [a-zA-Z]
ALNUM      {ALPHA}|{DIGIT}

INT        {DIGIT}+
ID         {ALPHA}{ALNUM}*


%%

"("  {return OPENPAR;}
")"  {return CLOSEPAR;}
"*"  {return MULT;}
"/"  {return DIV;}
"+"  {return PLUS;}
"-"  {return MINUS;}
"="  {return EQ;}
"!=" {return NEQ;}
">"  {return GT;}
"<"  {return LT;}
">=" {return GEQ;}
"<=" {return LEQ;}
"{"  {return OPENCURLY;}
"}"  {return CLOSECURLY;}
";"  {return SEMICOLON;}
","  {return COMMA;}


Integer {return INTEGER;}
String  {return STRING;}
if      {return IF;}
while   {return WHILE;}
return  {return RETURN;}
print   {return PRINT;}
else  {return ELSE;}

{ALPHA}{ALNUM}* {return IDENT;}

\"[^"]*\"       {return STRINGLIT;}

{INT}        {return INTLIT;}


#[^\n]* { /* Eat comments */}


[\n]    {yycol = 0;}

[\r\t ] { /* Skip  whitespace*/ }


    /* Everything else is an invalid character. */
.      {return ERROR;}

%%



int main( int argc, char **argv )
{
   enum yytokentype tok = 0;

   ++argv, --argc;  /* skip over program name */
   if ( argc > 0 )
      yyin = fopen( argv[0], "r" );
   else
      yyin = stdin;

   while((tok = yylex())!=EOFTok)
   {
      print_tok(tok);
   }
}
